@@ -4,8 +4,9 @@
  *  Al Longyear <longyear@netcom.com>
  *
  *  Dynamic PPP devices by Jim Freeman <jfree@caldera.com>.
+ *  ppp_tty_receive ``noisy-raise-bug'' fixed by Ove Ewerlid <ewerlid@syscon.uu.se>
  *
- *  ==FILEVERSION 2==
+ *  ==FILEVERSION 3==
  *
  *  NOTE TO MAINTAINERS:
  *     If you modify this file at all, increment the number above.
@@ -55,7 +56,7 @@
 #endif
 
 #ifndef PPP_MAX_DEV
-#define PPP_MAX_DEV	INT_MAX
+#define PPP_MAX_DEV	256
 #endif
 
 /* $Id: ppp.c,v 1.5 1995/06/12 11:36:53 paulus Exp $
@@ -69,10 +70,11 @@
 #ifdef MODULE
 #include <linux/module.h>
 #include <linux/version.h>
-#define STATIC static
+#define STATIC
 #else
 #define MOD_INC_USE_COUNT do { } while(0)
 #define MOD_DEC_USE_COUNT do { } while(0)
+#define STATIC static
 #endif /* def MODULE */
 
 #include <endian.h>
@@ -141,7 +143,7 @@ static void ppp_init_ctrl_blk (register struct ppp *);
 static void ppp_kick_tty (struct ppp *, struct ppp_buffer *bfr);
 static int ppp_doframe (struct ppp *);
 static struct ppp *ppp_alloc (void);
-static void ppp_print_buffer (const u_char *, u_char *, int);
+static void ppp_print_buffer (const u_char *, const u_char *, int);
 extern inline void ppp_stuff_char (struct ppp *ppp,
 				   register struct ppp_buffer *buf,
 				   register u_char chr);
@@ -166,11 +168,17 @@ static int  rcv_proto_ccp (struct ppp *, u_short, u_char *, int);
 #define OPTIMIZE_FLAG_TIME	0
 #endif
 
+#ifndef PPP_MAX_DEV
+#define PPP_MAX_DEV 256
+#endif
+
 /*
  * Parameters which may be changed via insmod.
  */
-       int  ppp_flag_time = OPTIMIZE_FLAG_TIME;
-       int  ppp_max_dev   = PPP_MAX_DEV;
+
+STATIC int  flag_time = OPTIMIZE_FLAG_TIME;
+STATIC int  max_dev   = PPP_MAX_DEV;
+
 /*
  * The "main" procedure to the ppp device
  */
@@ -227,7 +235,7 @@ static int ppp_dev_rebuild (void *, struct device *, unsigned long,
 
 static int ppp_tty_read (struct tty_struct *, struct file *, u_char *,
 			 unsigned int);
-static int ppp_tty_write (struct tty_struct *, struct file *, u_char *,
+static int ppp_tty_write (struct tty_struct *, struct file *, const u_char *,
 			  unsigned int);
 static int ppp_tty_ioctl (struct tty_struct *, struct file *, unsigned int,
 			  unsigned long);
@@ -236,7 +244,7 @@ static int ppp_tty_select (struct tty_struct *tty, struct inode *inode,
 static int ppp_tty_open (struct tty_struct *);
 static void ppp_tty_close (struct tty_struct *);
 static int ppp_tty_room (struct tty_struct *tty);
-static void ppp_tty_receive (struct tty_struct *tty, u_char * cp,
+static void ppp_tty_receive (struct tty_struct *tty, const u_char * cp,
 			     char *fp, int count);
 static void ppp_tty_wakeup (struct tty_struct *tty);
 
@@ -519,7 +527,7 @@ ppp_init_ctrl_blk (register struct ppp *ppp)
 	ppp->slcomp     = NULL;
 	ppp->read_wait  = NULL;
 	ppp->write_wait = NULL;
-	ppp->last_xmit  = jiffies - ppp_flag_time;
+	ppp->last_xmit  = jiffies - flag_time;
 
 	/* clear statistics */
 	memset (&ppp->stats, '\0', sizeof (struct pppstat));
@@ -809,17 +817,19 @@ ppp_tty_close (struct tty_struct *tty)
 {
 	struct ppp *ppp = tty2ppp (tty);
 
-	if (ppp == NULL || ppp->magic != PPP_MAGIC) {
-		if (ppp->flags & SC_DEBUG)
-			printk (KERN_WARNING
-				"ppp: trying to close unopened tty!\n");
-	} else {
-		CHECK_PPP_VOID();
-		if (ppp->flags & SC_DEBUG)
-			printk (KERN_INFO "ppp: channel %s closing.\n",
-				ppp2dev(ppp) -> name);
-		ppp_release (ppp);
-		MOD_DEC_USE_COUNT;
+	if (ppp != NULL) {
+		if (ppp->magic != PPP_MAGIC) {
+			if (ppp->flags & SC_DEBUG)
+				printk (KERN_WARNING
+				       "ppp: trying to close unopened tty!\n");
+		} else {
+			CHECK_PPP_VOID();
+			if (ppp->flags & SC_DEBUG)
+				printk (KERN_INFO "ppp: channel %s closing.\n",
+					ppp2dev(ppp) -> name);
+			ppp_release (ppp);
+			MOD_DEC_USE_COUNT;
+		}
 	}
 }
 
@@ -847,7 +857,7 @@ ppp_tty_open (struct tty_struct *tty)
 /*
  * Allocate the structure from the system
  */
-	ppp = ppp_alloc ();
+	ppp = ppp_alloc();
 	if (ppp == NULL) {
 		if (ppp->flags & SC_DEBUG)
 			printk (KERN_ERR
@@ -869,7 +879,7 @@ ppp_tty_open (struct tty_struct *tty)
 	if (tty->driver.flush_buffer)
 		tty->driver.flush_buffer (tty);
 /*
- * Allocate space for the default VJ header compression slots (16)
+ * Allocate space for the default VJ header compression slots
  */
 	ppp->slcomp = slhc_init (16, 16);
 	if (ppp->slcomp == NULL) {
@@ -967,9 +977,9 @@ ppp_tty_wakeup_code (struct ppp *ppp, struct tty_struct *tty,
  * transmission block.
  */
 			if (ppp2dev (ppp) -> flags & IFF_UP) {
-			        ppp2dev (ppp)->tbusy = 0;
+				if (xbuf->type == BUFFER_TYPE_DEV_WR)
+				        ppp2dev (ppp)->tbusy = 0;
 				mark_bh (NET_BH);
-				dev_tint (ppp2dev (ppp));
 			}
 /*
  * Wake up the transmission queue for all completion events.
@@ -978,18 +988,21 @@ ppp_tty_wakeup_code (struct ppp *ppp, struct tty_struct *tty,
 /*
  * Look at the priorities. Choose a daemon write over the device driver.
  */
+			cli();
 			xbuf = ppp->s1buf;
 			ppp->s1buf = NULL;
 			if (xbuf == NULL) {
 				xbuf = ppp->s2buf;
 				ppp->s2buf = NULL;
 			}
+			sti();
 /*
  * If there is a pending buffer then transmit it now.
  */
 			if (xbuf != NULL) {
 				ppp->flags &= ~SC_XMIT_BUSY;
 				ppp_kick_tty (ppp, xbuf);
+				return;
 			}
 		}
 	}
@@ -1014,12 +1027,11 @@ ppp_tty_wakeup (struct tty_struct *tty)
 	struct ppp_buffer *xbuf;
 	struct ppp *ppp = tty2ppp (tty);
 
-	if (!ppp || ppp->magic != PPP_MAGIC) {
-		if (ppp->flags & SC_DEBUG)
-			printk (KERN_DEBUG "PPP: write_wakeup called but "
-				"couldn't find PPP struct.\n");
+	if (!ppp)
+		return;
+
+	if (ppp->magic != PPP_MAGIC)
 		return;
-	}
 /*
  * Ensure that there is a transmission pending. Clear the re-entry flag if
  * there is no pending buffer. Otherwise, send the buffer.
@@ -1101,16 +1113,25 @@ ppp_tty_room (struct tty_struct *tty)
  */
 
 static void
-ppp_tty_receive (struct tty_struct *tty, u_char * data, char *flags, int count)
+ppp_tty_receive (struct tty_struct *tty, const u_char * data,
+		 char *flags, int count)
 {
 	register struct ppp *ppp = tty2ppp (tty);
-	register struct ppp_buffer *buf = ppp->rbuf;
+	register struct ppp_buffer *buf = NULL;
 	u_char chr;
+/*
+ * Fetch the pointer to the buffer. Be careful about race conditions.
+ */
+	if (ppp != NULL)
+		buf = ppp->rbuf;
+
+	if (buf == NULL)
+		return;
 /*
  * Verify the table pointer and ensure that the line is
  * still in PPP discipline.
  */
-	if (!ppp || ppp->magic != PPP_MAGIC) {
+	if (ppp->magic != PPP_MAGIC) {
 		if (ppp->flags & SC_DEBUG)
 			printk (KERN_DEBUG
 				"PPP: handler called but couldn't find "
@@ -1765,12 +1786,12 @@ ppp_tty_read (struct tty_struct *tty, struct file *file, u_char * buf,
 /*
  * Validate the pointer to the PPP structure
  */
-	if (!ppp || ppp->magic != PPP_MAGIC) {
-		if (ppp->flags & SC_DEBUG)
-			printk (KERN_ERR
-				"ppp_tty_read: cannot find ppp channel\n");
+	if (!ppp)
 		return -EIO;
-	}
+
+	if (ppp->magic != PPP_MAGIC)
+		return -EIO;
+
 	CHECK_PPP (-ENXIO);
 
 	if (ppp->flags & SC_DEBUG)
@@ -1950,10 +1971,10 @@ ppp_dev_xmit_lower (struct ppp *ppp, struct ppp_buffer *buf,
  */
 	buf->count = 0;
 
-	if (non_ip || ppp_flag_time == 0)
+	if (non_ip || flag_time == 0)
 		ins_char (buf, PPP_FLAG);
 	else {
-		if (jiffies - ppp->last_xmit > ppp_flag_time)
+		if (jiffies - ppp->last_xmit > flag_time)
 			ins_char (buf, PPP_FLAG);
 	}
 	ppp->last_xmit = jiffies;
@@ -2139,7 +2160,7 @@ send_revise_frame (register struct ppp *ppp, u_char *data, int len)
  */
 
 static int
-ppp_tty_write (struct tty_struct *tty, struct file *file, u_char * data,
+ppp_tty_write (struct tty_struct *tty, struct file *file, const u_char * data,
 	       unsigned int count)
 {
 	struct ppp *ppp = tty2ppp (tty);
@@ -2148,12 +2169,11 @@ ppp_tty_write (struct tty_struct *tty, struct file *file, u_char * data,
 /*
  * Verify the pointer to the PPP data and that the tty is still in PPP mode.
  */
-	if (!ppp || ppp->magic != PPP_MAGIC) {
-		if (ppp->flags & SC_DEBUG)
-			printk (KERN_ERR
-				"ppp_tty_write: cannot find ppp unit\n");
+	if (!ppp)
+		return -EIO;
+
+	if (ppp->magic != PPP_MAGIC)
 		return -EIO;
-	}
 
 	CHECK_PPP (-ENXIO);
 /*
@@ -2313,12 +2333,12 @@ ppp_tty_ioctl (struct tty_struct *tty, struct file *file, unsigned int param2,
 /*
  * Verify the status of the PPP device.
  */
-	if (!ppp || ppp->magic != PPP_MAGIC) {
-		if (ppp->flags & SC_DEBUG)
-			printk (KERN_ERR
-			"ppp_tty_ioctl: can't find PPP block from tty!\n");
+	if (!ppp)
 		return -EBADF;
-	}
+
+	if (ppp->magic != PPP_MAGIC)
+		return -EBADF;
+
 	CHECK_PPP (-ENXIO);
 /*
  * The user must have an euid of root to do these requests.
@@ -2608,12 +2628,12 @@ ppp_tty_select (struct tty_struct *tty, struct inode *inode,
 /*
  * Verify the status of the PPP device.
  */
-	if (!ppp || ppp->magic != PPP_MAGIC) {
-		if (ppp->flags & SC_DEBUG)
-			printk (KERN_ERR
-		       "ppp_tty_select: can't find PPP block from tty!\n");
+	if (!ppp)
 		return -EBADF;
-	}
+
+	if (ppp->magic != PPP_MAGIC)
+		return -EBADF;
+
 	CHECK_PPP (0);
 /*
  * Branch on the type of select mode. A read request must lock the user
@@ -3304,7 +3324,7 @@ ppp_alloc (void)
 		if (!set_bit(0, &ppp->inuse))
 			return (ppp);
 		ctl = ctl->next;
-		if (++if_num == ppp_max_dev)
+		if (++if_num == max_dev)
 			return (NULL);
 	}
 /*
@@ -3357,7 +3377,7 @@ ppp_alloc (void)
  */
 
 static void
-ppp_print_hex (register u_char * out, u_char * in, int count)
+ppp_print_hex (register u_char * out, const u_char * in, int count)
 {
 	register u_char next_ch;
 	static char hex[] = "0123456789ABCDEF";
@@ -3371,7 +3391,7 @@ ppp_print_hex (register u_char * out, u_char * in, int count)
 }
 
 static void
-ppp_print_char (register u_char * out, u_char * in, int count)
+ppp_print_char (register u_char * out, const u_char * in, int count)
 {
 	register u_char next_ch;
 
@@ -3390,7 +3410,7 @@ ppp_print_char (register u_char * out, u_char * in, int count)
 }
 
 static void
-ppp_print_buffer (const u_char * name, u_char * buf, int count)
+ppp_print_buffer (const u_char * name, const u_char * buf, int count)
 {
 	u_char line[44];
 
